package esmith::AD::Group;

use strict;
use base 'esmith::AD';
use esmith::AD::User;
no warnings ('qw');

=head1 NAME

esmith::AD::Group -- Sub-Class to the Koozali esmith::AD API

=head1 SYNOPSIS

    use esmith::AD::Group;
    
    my $ad = esmith::AD::Group->new();
    my @groups = $ad->listGroups;

    
=head1 DESCRIPTION

This module provide esmith::AD::Group methods for management of users 
stored in the Koozali SME Server Active Directory.  These methods are 
the preferred methods to interact with user objects in the Active 
Directory.  Many methods in this class inherit functionality from the 
esmith::AD parent class and provide run-time binding of LDAP query 
elements to these parent methods.  Methods provided in this module 
abstract Net::LDAP so that knowledge of this module is not required.

=cut

sub new {
    my ($class, %params) = @_;

    #Define runtime LDAP query parameters
    my $base = {
                'default' => '&(objectClass=group)(!(objectClass=computer))'
               };
    my $set  = {
                         'all'       => '',
                'builtin' => '(!(objectClass=posixAccount))',
                'koozali' => '(objectClass=posixAccount)',
                         'single'    => ''
                        };
    my $type = {
                 'posix'   => ['top','group','posixAccount']
               };

    my $attribute = {
                      'default' => 'sAMAccountName',
                      'dn'      => 'DN'
                    };
    my $self = {
                'base'      => $base,
                'set'       => $set,
                'attribute' => $attribute,
                'type'      => $type
               };

    #Bless parameters into the class
    bless ($self, $class);
    
    return ($self);
}

=head2 Methods


=head3 addUserToGroup('groupname','username')

This method adds a user to a group given a username and group name.  
The method returns 1 of successful and 0 if something went wrong. 

Usage:  $ad->addUserToGroup('groupname','username') 

=cut

sub addUserToGroup {
    my ($self,$group,$user) = @_;
    return 0 unless ($user && $group);

    my $adU = esmith::AD::User->new();
    return 0 unless ($adU->doesUserExist($user));

    #Verify Group exists so we don't run a bunch of bogus AD queries.
    return 0 unless ($self->doesGroupExist($group)); 

    my $groupMembers = $self->getGroupAttr($group,'member','all') || '';
    unless ($groupMembers) {$groupMembers = [];}
 
    my $memberAddDN = $self->getObjectDN($user); 
    
    push(@$groupMembers, $memberAddDN) 
      unless grep{$_ == $memberAddDN} @$groupMembers;
    
    if ($self->setAttr($group,'member',$groupMembers)) {return(1);}
    else {return (0)}

}


=head3 attributes()

This method returns a list of Active Directory group attributes managed
by this API.

Usage:  @ad_group_attributes = $ad->attributes();

=cut

sub attributes {
    my $self = shift;
    
    my %dataHash = $self->validData;

    return (keys %dataHash);
}

=head3 createGID($groupname)

This method calculates a unix GID for an Active Directory group
based upon the SID for the Active Directory object.

Usage:  my $gid = $ad->createGID($groupname);

=cut

sub createGID {
    my ($self,$group) = @_;

    return (0) unless $group;   
    
    #Pull SID from the AD   
    return (0) unless (my $SID = $self->getSID($group));
   
    #Return GID = RID + 4000    
    my $RID = (split(/-/,$SID))[7];
    return($RID+4000);
}   


=head3 doesGroupExist($groupname)

This method checks the Active Directory for the existance of a group name 
passed as a parameter to the method.  The method returns 1 if the 
group name exists in the Active Directory and 0 if it does not.

Usage:  $group_check = $ad->doesGroupExist('group_name');

=cut

sub doesGroupExist {
    my ($self,$group) = @_;
   
    return 0 unless $group; 
 
    my ($groupRecord) = $self->queryObjects($group); 

    if(defined $groupRecord) {return (1);}
    else{return(0);}
}


=head3 getGID($groupname)

This method returns the unix GID from the Active Directory, 
given a groupname.

Usage:  my $gid = $ad->getGID('group_name');

=cut

sub getGID { 
    my ($self,$group)=@_;

    return 0 unless $group;
    
    my ($record) = $self->queryObjects($group);
    die $self->error("Unable to find \"$group\" in the Active Directory.\n")
      unless ($record);
    
    my $GID = $record->get_value('gidNumber') || '';
       
    die $self->error("Unable to retrieve GID for $group\n")
      unless ($GID);
    
    return($GID);
}   

=head3 getGroupAttr($group,$attr,$set)

This method returns the value of the specified attribute for a group
listed in the Active Directory. Some attributes could be multi-value
attributes.  By default this method will return the first value of a 
multi-value attribute in scalar contact unless you tell the method to
return all value.

Usage:  $single_value = $ad->getGroupAttr('group','attribute');
        $multi_value_ref = $ad->getGroupAttr('group',attribute','all');

=cut

sub getGroupAttr {

    my($self,$group,$attr,$set)=@_;

    return 0 unless $group;

    #Check that the search attribute is supported
    die $self->error("Attribute \"$attr\" not supported\n")
      unless ($self->validData->{$attr});
   
    #Query AD
    my ($record) = $self->queryObjects($group);
    die $self->error("Unable to find \"$group\" in Active Directory.\n")
      unless ($record);

    #Pull attribute value single or all
    if ($set eq 'all') 
    {
        my $valueRef = $record->get_value($attr,asref => 1) || '';
        return ($valueRef);
    }
    else 
    {
        my $value = $record->get_value($attr) || '';
        return ($value);
    }
}

=head3 getMembers($group)

This method returns a list of members of a specified group.  The user
needs to be aware of what he is asking this method to report because 
Active Directory allows nested groups (i.e., groups that are members of
a group).  In most cases, if you ask this method for members of a group
designed by this API, then it will return a list of users that are members
of the designed group.  In the future, we may add logic to delinate user 
members versus nested group members, but for now we are going to be lazy 
and just report what is in Active Directory.   

Usage:  @group_mambers = $ad->getMembers('group_name');

=cut

sub getMembers {
    my($self,$group)=@_;

    return (0) unless $group;

    #Get group members as an array ref
    my $memberDN = $self->getGroupAttr($group,'member','all') || '';
   
    #AD returns the DN for group members, so we need to generate names
    my ($sam,@members);  
    foreach (@$memberDN) {
       $sam = $self->getName($_) || '';
       push (@members,$sam) unless ($sam eq ''); 
    }
    return (@members);
}

=head3 groupObjects($set)

This method returns a list of Active Directory group objects given one
of the pre-defined sets:  all, koozali, builtin.   To retreive a single
object, pass the object sAMAccount name to the method.

Objects returned from this method are Net::LDAP record objects.  To access
object attribute values, use the Net::LDAP::Record->get_value() method.
Example:  $record->get_value('cn').

Usage:
       use esmith::AD::Group;

       $ad = esmith::AD::Group->new();
       @koozaliGroups = $ad->groupObjects('koozali');
=cut

sub groupObjects
{
    my ($self,$set) = @_;

    #Return all user objects if set isn't specified
    $set = 'all' unless ($set);
    
    #Run query
    my @records = $self->queryObjects($set);

    if (length(@records) == 0)
    {
        die $self->error("Unable to query objects from Active Directory.\n")
    }
    elsif (length(@records) == 1)
    {
        return($records[0]);
    }
    else
    {
        return(@records);
    }
}


=head3 listGroups($type)

This method returns a list of groups based upon the type of group 
specified as a parameter to the method.  Valid types include:

 - builtin:  Groups defined during the Active Directory 
             provisioning (e.g., Administrator);
 - koozali:  Groups defined in the Koozali API.
 - all:  All groups.

If the method doesn't receive a valid type, then it will return a 
list of all groups in the Active Directory

Usage:
       use esmith::AD::Group;
    
       my $ad = esmith::AD::Group->new();
       my @groups = $ad->listGroups('buildin');
=cut

sub listGroups {
    my ($self, $set) = @_;

    return($self->listObjects($set)); 
}


=head3 removeUserFromGroup()

This method removes a user from a group given a username and group name.
The method returns 1 of successful and 0 if something went wrong.

Usage:  $ad->removeUserFromGroupp('username','groupname')

=cut

sub removeUserFromGroup {
    my ($self,$user,$group) = @_;
    return 0 unless ($user && $group);

    my $adU = esmith::AD::User->new();
    return 0 unless ($adU->doesUserExist($user));

    #Verify Group exists so we don't run a bunch of bogus AD queries.
    return 0 unless ($self->doesGroupExist($group));

    my $groupMembers = $self->getGroupAttr($group,'member','all') || '';
    return (1) unless ($groupMembers); #group is empty already

    my $memberRemoveDN = $self->getObjectDN($user);

    my @newMembers = grep {$_ ne $memberRemoveDN} @$groupMembers;
    
    if ($self->setAttr($group,'member',\@newMembers)) {return(1);}
    else {return (0)}
    
}


=head3 setGroupAttr($group,$attr,$value)**

This method sets or modifies an attribute value for the specified 
groupname.

Usage:  $ad->setGroupAttr('groupname','attribute','value')

=cut 

sub setGroupAttr {
    my ($self,$group,$attr,$value) = @_;
   
    return 0 unless $group;
   
    if ($self->setAttr($group,{$attr=>$value})) { return(1); }
    else { 
        die->error("Unable to modify \"$attr\" for groupname \"$group\".\n");
    }
}

=head3 setPosixGroup ($group)

This method designates a group as a Posix Group.  The Posix group type
represents the groups managed by Koozali and are those groups that are 
displayed in the server-manager

Usage:  $ad->setPosixGroup('groupname')

=cut

sub setPosixGroup { 
    my ($self,$group) = @_;

    return 0 unless $group; 

    return($self->setPosix($group));
}

=head3 validData()

This method returns a hash reference of valid User attributes and value formats.
Value formats are qr// quoted regex stored in the hash values.

Usage:  $hashRef = $ad->validData()

=cut

sub validData {
    my $self = shift;

    return ({
               'member'         => 1,
               'description'	=> 1,
               'gidNumber'	=> 1	             
           });
}

=head1 COPYRIGHT

Copyright (c) 2016 Koozali Foundation, Inc.
This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 AUTHOR

Greg Zartman, Koozali Foundation <gzartman@koozali.org>

=head1 SEE ALSO

Net::LDAP, esmith::AD

=cut

1;
