package esmith::AD::User;

use strict;
use base 'esmith::AD';
no warnings ('qw');
use Data::Dumper;

=head1 NAME

esmith::AD::User - Sub-Class to the Koozali esmith::AD API

=head1 SYNOPSIS

    use esmith::AD::User;
   
    my $ad = esmith::AD::User->new();
    my @users = $ad->listUsers;
 
 
=head1 DESCRIPTION

This module provide esmith::AD::User methods for management of users stored
in the Koozali SME Server Active Directory.  These methods are the preferred 
methods to interact with user objects in the Active Directory.  Many methods 
in this class inherit functionality from the esmith::AD parent class and 
provide run-time binding of LDAP query elements to these parent methods. 
Methods provided in this module abstract Net::LDAP so that knowledge of 
this module is not required.
=cut

sub new 
{
    my ($class, %params) = @_;
    
    #Define runtime LDAP query parameters
    my $base = {
                'default' => '&(objectClass=user)(!(objectClass=computer))'
               };
    my $set  = {
                'all'      => '',
                'builtin'  => '(!(objectClass=posixAccount))',
                'koozali'  => '(objectClass=posixAccount)',
                'single'   => ''
               };
    my $type = {
                'posix'    => ['top','person','organizationalPerson',
			       'user','posixAccount','smeExtended']
               };
    my $attribute = {
                      'default' => 'sAMAccountName',
                      'status'  => 'userAccountControl'
                    };
    my $self = {
                'base'      => $base,
                'set'       => $set,
                'attribute' => $attribute,
                'type'      => $type
               };

    
    #bless paramters into the class
    bless ($self, $class);

    return ($self);
}

=head2 Methods

=head3 activeUsers()

This method returns the total number of Koozali active users.  These are users 
created with this API and do not include buildin users such as admin. 

Usage:  
        my $active_users = $ad->activeUsers();

=cut

sub activeUsers 
{
    my $self = shift;

    #Build query filter to search for active koozali users
    my $queryFilter = $self->{'base'}->{'default'} . 
                      $self->{'set'}->{'koozali'} .
                      '(|(userAccountControl=512)(userAccountControl=66048))';
    
    #Run query 
    my @records = $self->runQuery($queryFilter);
    
    my $activeUsers = @records;

    return($activeUsers); 
}


=head3 attributes()

This method returns a list of Active Directory user attributes managed
by this API.

Usage:  
        my @ad_user_attributes = $ad->attributes();

=cut

sub attributes 
{
    my $self = shift;

    my $dataRef = $self->validData();
    
    return (keys %$dataRef);
}

=head3 createAutoPseudonyms($username)

This method creates dot and bar pseudonyms for a user and then 
adds them as an attribute to the user object.  

Usage:  
        $ad->createAutoPseudonyms($username);

=cut

sub createAutoPseudonyms 
{
    my ($self,$user) = @_;

    return 0 unless $user;

    #Query AD
    my ($record) = $self->queryObjects($user);
    die $self->error("Unable to find \"$user\" in the Active Directory.\n")
      unless ($record);

    #Pull first and last name.  Exit if either isn't set, as we can't 
    #build a pseudonym without both. 
    my $firstName = $record->get_value('givenName') || '';
    my $lastName = $record->get_value('sn') || '';
    return (0) if ($firstName eq '' || $lastName eq ''); 

    #Get ProxyAddresses attribute as list reference or create it
    my $proxyAddresses = $record->get_value('proxyAddresses');
    unless (ref($proxyAddresses) eq 'ARRAY') {$proxyAddresses = [];}

    my $dotPseudonym = 'sme:' . $firstName . '.' . $lastName;
    push(@$proxyAddresses, $dotPseudonym)
      unless grep{$_ eq $dotPseudonym} @$proxyAddresses;
    
    my $barPseudonym = 'sme:' . $firstName . '_' . $lastName;
    push(@$proxyAddresses, $barPseudonym)
      unless grep{$_ eq $barPseudonym} @$proxyAddresses;

    if ($self->setAttr($user,'proxyAddresses',$proxyAddresses)) 
    {
        return(1);
    }
    else {return (0)}
}

    
=head3 deleteAutoPseudonyms($username)

This method creates dot and bar pseudonyms for a user and then 
adds them as an attribute to the user object.  

Usage:  


=cut

sub deleteAutoPseudonyms 
{
    my ($self,$user) = @_;

    return 0 unless $user;

    #Query AD
    my ($record) = $self->queryObjects($user);
    die $self->error("Unable to find \"$user\" in the Active Directory.\n")
      unless ($record);

    #Pull first and last name.  Exit if either isn't set, as we can't 
    #build auto pseudonyms without both. 
    my $firstName = $record->get_value('givenName') || '';
    my $lastName = $record->get_value('sn') || '';
    return (0) if ($firstName eq '' || $lastName eq ''); 

    #Build array ref of auto pseudonyms 
    my $autoPseudonyms = ['sme:' . $firstName . '.' . $lastName,
                          'sme:' . $firstName . '_' . $lastName
                         ];
 
    if ($self->deleteAttr($user,{'proxyAddresses'=>$autoPseudonyms})) 
    {
        return(1);
    }
    else {return (0)}
}
 

    
=head3 createUID($username)

This method calculates a unix UID for an Active Directory user 
based upon the SID for the Active Directory object.

Usage:  
       my $uid = $ad->createUID($username);

=cut

sub createUID {
    my ($self,$name) = @_;
    
    return 0 unless $name;   
    
    #Pull SID from the AD   
    return (0) unless (my $SID = $self->getSID($name)); 
   
    #Return UID = RID + 4000    
    my $RID = (split(/-/,$SID))[7];
    return($RID+4000);
}

=head3 doesUserExist($username)

This method checks the Active Directory for the existance of a username passed
as a parameter to the method.  If the username exists, this method returns a 
hash reference of the attribute/value pairs for the queried user.  Otherwise, 
this method returns 0. 

Usage: 
       use esmith::AD::User;

       my $ad = esmith::AD::User->new();
       my $userData = $ad->doesUserExist('user_name');

       if ($userData) 
       {
           print "User exists.  His first name is $UserData->{'givenName'}";
       }

=cut

sub doesUserExist {
    my ($self,$user) = @_;

    return 0 unless $user;
    
    my $singleRecord = '';
    ($singleRecord) = $self->queryObjects($user);
 
    if (defined $singleRecord) 
    {
        my @attributes = $self->attributes();
        my %record = map {$_ => ($singleRecord->get_value($_))||''} @attributes;
        return(\%record);
    }
    else {return(0);}
}


=head3 doesPseudonymExist($pseudonym)

This method checks the Active Directory for the existance of a pseudonym.  
If the pseudonym exists, this method returns a hash reference of the 
attribute/value pairs for the user associated with this pseudonym.  
Otherwise, this method returns 0.  This method is useful for performing 
pseudonym clash checks.

Usage: 
       use esmith::AD::User;

       my $ad = esmith::AD::User->new();
       my $userData = $ad->doesPseudonymExist('pseudonym');

       print "User $userData{'sAMAccountName'} already has this pseudonym.\n"
           if ($UserData);

=cut

sub doesPseudonymExist {
    my ($self,$rawPseudonym) = @_;

    return 0 unless $rawPseudonym;
    
    my $pseudonym = 'sme:' . $rawPseudonym;
    my ($singleRecord) = 
      $self->queryObjects('all','proxyAddress',$pseudonym) || undef;
    
    if (defined $singleRecord) 
    {
        my @attributes = $self->attributes();
        my %record = map {$_ => ($singleRecord->get_value($_))||''} @attributes;
        return(\%record);
    }
    else {return(0);}
}


=head3 getUID($username)

This method returns the unix UID for a user from the Active Directory.

Usage: 
       my $uid = $ad->getUID('username'); 

=cut

sub getUID 
{
    my ($self,$user)=@_;

    return 0 unless $user; 

    my ($record) = $self->queryObjects($user);

    die $self->error("Unable to find \"$user\" in the Active Directory.\n")
      unless ($record);

    my $UID = $record->get_value('uidNumber') || '';

    die $self->error("Unable to retrieve UID for $user\n") 
      unless ($UID);
    
    return($UID);
}

=head3 getUserAttr($username,$attr,$set)

This method returns the value of the specified attribute for a user
listed in the Active Directory.  Some attributes could be multi-value
attributes.  By default this method will return the first value of a
multi-value attribute in scalar contact unless you tell the method to
return all values.  If all values are specified, this method returns 
an array reference to the list of values. 

Usage:  
        $single_value = $ad->getUserAttr('username','attribute');
        $multi_ref = $ad->getGroupAttr('username','attribute','all');

=cut

sub getUserAttr 
{
    my($self,$user,$attr,$set)=@_;

    return 0 unless ($user && $attr);
    $set = 'single' unless $set;  

    #Check that the search attribute is supported
    die $self->error("Attribute \"$attr\" not supported.\n")
      unless ($self->attributes($attr));

    #Query AD
    my ($record) = $self->queryObjects($user);
    die $self->error("Unable to find \"$user\" in the Active Directory.\n")
      unless ($record);
   
    #Pull attribute value single or all
    if ($set eq 'all') 
    {
        my $valueRef = $record->get_value($attr) || '';
        if (ref ($valueRef) eq "ARRAY") {return($valueRef);}
        else {return([$valueRef]);}
    }
    else 
    {
        my $value = $record->get_value($attr) || '';
        return ($value);
    }
}


=head3 getUserGroups($username)

This returns a list of groups the user is a member of.

Usage:  
       @user_groups = $ad->getUserGroups('username');

=cut

sub getUserGroups 
{
    my($self,$user)=@_;

    return (0) unless $user;
    
    #Get getups
    my $memberDN = $self->getUserAttr($user,'memberOf','all') || '';
   
    #AD returns the DN for group members, so we need to generate names
    my ($sam,@groups);
    foreach (@$memberDN) 
    {
       $sam = $self->getName($_) || '';
       push (@groups,$sam) unless ($sam eq '');
    }
    return (@groups);
}


=head3 getUserStatus($username)

This method returns user account status, enabled or disabled, for a 
specified username.

Usage:  
       $user_account_status=$ad->getUserStatus('username');

=cut

sub getUserStatus 
{
    my ($self,$user)=@_;

    return (0) unless $user; 

    return ($self->getStatus($user));
}


=head3 homeDir($username)

This method returns the home directory for a user given the the
username.

Usage:  
       $home_dir=$ad->homeDir('username');

=cut

sub homeDir 
{
    my ($self,$user)=@_;

    return (0) unless $user; 

    return($self->getUserAttr($user,'unixHomeDirectory'));
}


=head3 isUserInGroup($username,$group)

This method returns true a user exists in a group, otherwise it returns
false. 

Usage:  $is_user_in_group = $ad->isUserInGroup('username','groupname');

=cut

sub isUserInGroup 
{
    my ($self,$user,$group)=@_;

    return (0) unless ($user && $group);

    my @groups = $self->getUserGroups($user);
    
    if (grep {$_ eq $group} @groups) {return(1);}
    else {return(0);} 
}


=head3 listPseudonyms

This method returns a list all Koozali User pseudonyms in the Active Directory
including the admin account.  

Usage:    my @pseudonyms = $ad->listPseudonyms();

=cut

sub listPseudonyms 
{
    my ($self) = @_;

    #Pull all Koozali accounts
    my @adRecords = $self->queryObjects('koozali');

    my (@proxyAddresses, @pseudonyms) = ();
    foreach (@adRecords) 
    {
        @proxyAddresses = $_->get_value('proxyAddresses');
        push(@pseudonyms, (grep(!/sme/,(map{split/:/,$_}@proxyAddresses))));            
        @proxyAddresses = ();
    }
    return(@pseudonyms);
}


=head3 listUsers($set)

This method returns a list of usernames in the Active Directory
depending upon the search set sent to the method.  Valid search 
sets include:
 
 - builtin :  Users defined during the Active Directory 
              provisioning (e.g., Administrator);
 - koozali :  Users defined in the Koozali API.
 - all     :  All users (default).

Usage: 
       use esmith::AD::User;
   
       my $ad = esmith::AD::User->new();
       my @users = $ad->listUsers('koozali');
=cut

sub listUsers 
{
    my ($self,$set) = @_;
    
    return($self->listObjects($set)); 
}


=head3 setPosixUser($user)

This method designates a user as a Posix User.  The Posix user type
represents the users managed by Koozali and displayed in the 
server-manager.  This method returns 1 if successful and 0 if something
went wrong. 

Usage: $ad = esmith::AD::User;  
       $ad->setPosixUser('username')

=cut 

sub setPosixUser 
{
    my ($self,$name) = @_;

    return 0 unless $name; 
   
    return($self->setPosix($name));
}


=head3 setUserAttr($user,$attr,$value) 

This method sets or modifies user attribute/value pairs.  Attributes/value
pairs can be passed individually by passing a supported attribute name and 
the value as parameters to the method.   Multiple attribute/value pairs 
can be set or updated by passing a hash reference of these attribute/value
pairs to the $attr parameter.  If a hash reference is passed to the $attr
parameter, then the $value parameter does not need to be set.   

Usage
      #Single Value -- Update first name
      use esmith::AD::User;

      $ad = esmith::AD::User->new();
      $ad->setUserAttr('username','givenName','frank')


      #Multiple Values
      use esmith::AD::User;

      $ad = esmith::AD::User->new();
     
      $updates = {'givenName' => 'frank',
                  'sn         => 'smith',
                  'city'      => 'Timbuktu'
                 };
      $ad->setUserAttr('username',$updates);
 
=cut 

sub setUserAttr 
{
    my ($self,$name,$attr,$value) = @_;

    return 0 unless ($name && $attr); 
 
    if (ref($attr) eq 'HASH') 
    {
        $self->setManyAttr($name,$attr);
    }
    else 
    {
        $self->setManyAttr($name,{$attr=>$value});
    }
}


=head3 userObjects($set)

This method returns a list of Active Directory user objects given one 
of the pre-defined sets:  all, koozali, builtin.   To retreive a single
object, pass the object sAMAccount name to the method.

Objects returned from this method are Net::LDAP record objects.  To access
object attribute values, use the Net::LDAP::Record->get_value() method.  
Example:  $record->get_value('cn'). 

Usage:
       use esmith::AD::User;
       
       $ad = esmith::AD::User-new();

       @koozaliUsers = $ad->userObjects('koozali');
       foreach (@koozaliUsers) 
       {
           print "First Name = " . $_->get_value('first') . " \n";
       }


=cut

sub userObjects
{
    my ($self,$set) = @_;

    #Return all user objects if set isn't specified
    $set = 'all' unless ($set); 

    ##Run query
    my @records = $self->queryObjects($set);

    if (scalar(@records) == 0)
    {
        die $self->error("Unable to query objects from Active Directory.\n")
    }
    else
    {
        return(@records);
    }
}


=head3 userRecord('username') 

This method returns a hash reference of attribute/value pairs for a given
user in the Active Directory.  Attributes with multiple values will be stored
as an array reference in the hash value.  

Usage:
       use esmith::AD::User;
       
       $ad = esmith::AD::User-new();
       
       $userRecord = $ad->UserRecord('someUser'); 
       $firstName = $userRecord->{'givenName'};


=cut

sub userRecord
{
    my ($self,$user) = @_;

    return () unless $user; 

    #Run query
    my ($singleObject) = $self->userObjects($user);

    die "Unable to query user $user from Active Directory"
        unless $singleObject;

    #Build a hash of support attributes and values for user
    my @attributes = $self->attributes(); 
    my %record = map {$_ => ($singleObject->get_value($_))||''} @attributes;
   
    return(\%record); 
}


=head3 validData()

This method returns a hash reference of valid User attributes supported by
this class, which include:
	       
 	    cn                  =    Username
            company             =    Company Name
            countryCode         =    Country Code
            department          =    Department
            description
            displayName
            gidNumber
            givenName
            initials
            l                       =    City
            lockable
            loginShell
            mail
            MaxBlocks
            MaxBlocksSoftLim
            name
            postalCode
            preferredDeliveryMethod = 
            proxyAddresses
            removeable
            sAMAccountName          =   Username
            sn
            streetAddress           =   Street Address
            telephoneNumber
            title
            uidNumber
            unixHomeDirectory
            wWWHomePage

Attributes are stored as keys and all values are set to 1.  Future versions 
of this method may include regex formatting checks for each attribute. 

Other variable attributes can be added to the validData() method as long as 
they are valid attributes in the Active Data schema.  Details about Active 
Directory attributes can be found at the following URL:
 http://www.selfadsi.org/user-attributes.htm

Usage:  
       $hashRef = $ad->validData()

=cut 

sub validData 
{
    my $self = shift;
    ##TO DO:   We can possibly setup data checking by putting regex checks 
    #          as has values using the qr// quoted regex 
    return ({
               'cn'                         => 1,
               'company'                    => 1,
               'countryCode'                => 1,
               'department'                 => 1,
               'description'                => 1,
               'displayName'                => 1,
               'emailForward'               => 1,
               'gidNumber'                  => 1,
               'givenName'                  => 1,
               'homeDirectory'              => 1,
               'initials'                   => 1,
               'l'                          => 1,
               'lockable'                   => 1,
               'loginShell'                 => 1,
               'mail'                       => 1,
               'MaxBlocks'                  => 1,
               'MaxBlocksSoftLim'           => 1,
               'name'                       => 1,
               'postalCode'                 => 1,
               'proxyAddresses'             => 1,
               'removeable'                 => 1,
               'sn'                         => 1,
               'sAMAccountName'             => 1,
               'streetAddress'              => 1,
               'telephoneNumber'            => 1,
               'title'                      => 1,
               'uidNumber'                  => 1,
               'unixHomeDirectory'          => 1,
               'wWWHomePage'                => 1
            });
}


=head1 COPYRIGHT

Copyright (C) 2016 Koozali Foundation, Inc.
This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 AUTHOR

Greg Zartman, Koozali Foundation <gzartman@koozali.org>

=head1 SEE ALSO

Net::LDAP
esmith::AD

=cut

1;
